function [bmix]=mixed_approx_ws(a, np, ns)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Working version of mixed polynomial-rational approximation
% of the psi_1 function. This version with scaling.
% Last modification on Sept 16, 2019.
%
% Input data (follows notation in the manuscript):
%
% a  is the test matrix; 
% np is half the degree of Taylor polynomial (p_n in the manuscript),
%    except when np=0, in which case the degree is 1, 
% ns is the number of terms in the rational approximation, that is, 
%    half the degree of the rational part.
%
% Output.
%
% bmix is the approximation of psi_1(a) given by mixed approximation,
%      with scaling.
% 
%
% Note that the "total" degree (to be used for comparison with 
% purely polynomial approximations) is 2np + 2ns if np > 0 
% and 2ns + 1 if np = 0.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n=size(a,1);

%%%%%%%%%% Scale variable:   %%%%%%%%%%%%%
s=0;
while norm(a,'inf')>8 % or some other threshold
    a=a/2;
    s=s+1;
end
disp('Scaling with s=')
disp(s)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute mixed approximation.
% Polynomial part:
if np>0
    xx=bernoulli(0:2*np); 
    bpoly=eye(n); 
    aa=a; 
    for j=2:2*np+1
        bpoly=bpoly+(xx(j)/factorial(j-1))*aa; 
        aa=aa*a;
    end
else
    bpoly=eye(n)-a/2;
end

% Rational part:
brat=zeros(n);
ap2=(a/(2*pi))^2;
for k=1:ns
    brat=brat+inv(k*k*eye(n)+ap2)/(k^(2*np));
end
brat=brat*2*(-1)^np*ap2^(np+1);

% Mixed approximation:
bmix=bpoly+brat;

%%%%%%%%% Squaring: %%%%%%%%%%%%%%%%%%%%%%%
for k=1:s
    bmix=2*bmix*bmix/(a+2*bmix);
    a=2*a;
end
    


