function pmix=two_mixed_approx(a, v, np, ns)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Mixed polynomial-rational approximation
% of psi_1(A)*v. Uses 2x2 block structure.
% Last modification on Aug 24, 2020.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[n,~]=size(a);   % matrix size


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute mixed approximation.
% Polynomial part:
if np>0
    xx=bernoulli(0:2*np); 
    pmixpoly=v;
    aa=a; 
    for j=2:2*np+1
        pmixpoly=pmixpoly+(xx(j)/factorial(j-1))*aa*v;
        aa=aa*a;
    end
else
    pmixpoly=v-a*v/2;
end

% Rational part:
prat=zeros(n,1);
ap2=(a/(2*pi))^2;
for k=1:ns
   M=ap2+k*k*eye(n);
   A=M(1:n/2,1:n/2);
   B=M(1:n/2,n/2+1:n);
   C=M(n/2+1:n,1:n/2);
   D=M(n/2+1:n,n/2+1:n);
   % compute qs generators (A, C, D are symmetric tridiagonal, B is diagonal)
   pA=[0;ones(n/2-1,1)];
   qA=[diag(A,1).',0];
   aA=zeros(1,1,n/2);
   aA(:,:,1)=0; aA(:,:,n/2)=0;
   gA=[ones(n/2-1,1);0];
   hA=[0,diag(A,1).'];
   bA=zeros(1,1,n/2);
   bA(:,:,1)=0; bA(:,:,n/2)=0;
   dA=diag(A);
   pC=[0;ones(n/2-1,1)];
   qC=[diag(C,1).',0];
   aC=zeros(1,1,n/2);
   aC(:,:,1)=0; aC(:,:,n/2)=0;
   gC=[ones(n/2-1,1);0];
   hC=[0,diag(C,1).'];
   bC=zeros(1,1,n/2);
   bC(:,:,1)=0; bC(:,:,n/2)=0;
   dC=diag(C);
   pD=[0;ones(n/2-1,1)];
   qD=[diag(D,1).',0];
   aD=zeros(1,1,n/2);
   aD(:,:,1)=0; aD(:,:,n/2)=0;
   gD=[ones(n/2-1,1);0];
   hD=[0,diag(A,1).'];
   bD=zeros(1,1,n/2);
   bD(:,:,1)=0; bD(:,:,n/2)=0;
   dD=diag(D);
   
   S= D-(C/A)*B; % Schur complement
   % compute generators for S
   [pAi,aAi,qAi,gAi,bAi,hAi,dAi]=invert_187kk_function(pA,aA,qA,gA,bA,hA,dA);
   [pCA,qCA,aCA,gCA,hCA,bCA,dCA] = qs_product(pC,qC,aC,gC,hC,bC,dC,pAi,qAi,aAi,gAi,hAi,bAi,dAi);
   pCA=B*pCA;
   dCA=B*dCA;
   gCA=B*gCA;
   pS=[pD,-pCA];
   qS=[qD;qCA];
   for kk=1:n/2
       aS(:,:,kk)=[aD(:,:,kk),0,0;[0;0],aCA(:,:,kk)];
   end
   gS=[gD,-gCA];
   hS=[hD;hCA];
   for kk=1:n/2
       bS(:,:,kk)=[bD(:,:,kk),0,0;[0;0],bCA(:,:,kk)];
   end
   dS=dD-dCA;
   
   v1=v(1:n/2);
   v2=v(n/2+1:n);
   % compute w1 (first part of solution vector)
   % auxw2=A\v1;
   auxw2=qs_shifted_function(pA,qA,aA,gA,hA,bA,dA,0,v1);
   auxw=C*auxw2;
   % auxw=S\auxw;
   auxw=qs_shifted_function(pS,qS,aS,gS,hS,bS,dS,0,auxw);
   auxw=B*auxw;
   % auxw=A\auxw;
   auxw=qs_shifted_function(pA,qA,aA,gA,hA,bA,dA,0,auxw);
   w1=auxw+auxw2;
   % auxw=S\v2;
   auxw=qs_shifted_function(pS,qS,aS,gS,hS,bS,dS,0,v2);
   auxw=B*auxw;
   % auxw=-A\auxw;
   auxw=-qs_shifted_function(pA,qA,aA,gA,hA,bA,dA,0,auxw);
   w1=w1+auxw;
   % compute w2 (second part of solution vector)
   %auxw=A\v1;
   auxw=qs_shifted_function(pA,qA,aA,gA,hA,bA,dA,0,v1);
   auxw=C*auxw;
   % w2=-S\auxw;
   w2=-qs_shifted_function(pS,qS,aS,gS,hS,bS,dS,0,auxw);
   % auxw2=D\v2;
   auxw2=qs_shifted_function(pD,qD,aD,gD,hD,bD,dD,0,v2);
   auxw=B*auxw2;
   % auxw=S\auxw;
   auxw=qs_shifted_function(pS,qS,aS,gS,hS,bS,dS,0,auxw);
   auxw=C*auxw;
   % auxw=D\auxw;
   auxw=qs_shifted_function(pD,qD,aD,gD,hD,bD,dD,0,auxw);
   w2=w2+auxw+auxw2;
   prat=prat+[w1;w2]/(k^(2*np)); 
end
prat=(2*(-1)^np*ap2^(np+1))*prat;

% Mixed approximation:
pmix=pmixpoly+prat;
    
    



